(function() {
  var DOWN_ARROW, ESCAPE, MultiSelect, MultiSelectOptionList, MultiSelectValue, PropTypes, React, ReactDOM, SPACE, button, createClass, div, isEqual, makeGuid, map, ref, ref1;

  React = require('react');

  PropTypes = require('prop-types');

  createClass = require('create-react-class');

  ReactDOM = require('react-dom');

  map = require('lodash/map');

  isEqual = require('lodash/isEqual');

  MultiSelectValue = require('./multi_select_value');

  MultiSelectOptionList = require('./multi_select_option_list');

  makeGuid = require('../utils').makeGuid;

  ref = require('../constants/keyboard'), ESCAPE = ref.ESCAPE, DOWN_ARROW = ref.DOWN_ARROW, SPACE = ref.SPACE;

  ref1 = require('react-dom-factories'), div = ref1.div, button = ref1.button;


  /*
  Multi Select Props
  
  @props.options - REQUIRED - Array
    An array of options for the user to click - can be a flat array (where each entry is both the value and label)
    or an array of objects, where the value is the `valueField` and the label is the `labelField` (see below)
  
  @props.values - OPTIONAL - Array
    A flat array of values that the control initializes to. If the options array is objects, each entry should be the 
    `valueField` value. Will default to an empty array
  
  @props.labelField - OPTIONAL - String
    The attribute name from each option object that is the user facing label
  
  @props.valueField - OPTIONAL - String
    The attribute name from each option object that is the 'value`
  
  @props.returnFullObjects - OPTIONAL - Boolean - default: false
    Whether or not the values returned from the input are a collection of objects or a flat array of values
  
  @props.onChange - OPTIONAL - function
    Function/method to fire when the data changes
  
  @props.placeholder - OPTIONAL
    placeholder text for an empty control
  
  @props.tabIndex - OPTIONAL
    tab order of the edi button
  
  @props.onRemove - OPTIONAL - function
    function call when an item is removed, will pass the removed item
   */

  MultiSelect = createClass({
    displayName: 'MultiSelect',
    propTypes: {
      values: PropTypes.array,
      options: PropTypes.array,
      jsonPath: PropTypes.string,
      returnFullObjects: PropTypes.bool,
      disabled: PropTypes.bool,
      isInPopover: PropTypes.bool,
      labelField: PropTypes.string,
      valueField: PropTypes.string,
      tabIndex: PropTypes.number,
      placeholder: PropTypes.string,
      closeOnSelect: PropTypes.bool
    },
    contextTypes: {
      clearValidationError: PropTypes.func,
      addValidationError: PropTypes.func,
      getValidationStatus: PropTypes.func,
      toggleValidationError: PropTypes.func,
      openOverlay: PropTypes.func,
      closeOverlay: PropTypes.func
    },
    getDefaultProps: function() {
      return {
        values: [],
        options: [],
        returnFullObjects: true,
        isInPopover: false,
        disabled: false,
        labelField: 'label',
        valueField: 'value',
        defaultPlaceholder: 'Select from list...',
        closeOnSelect: false
      };
    },
    getInitialState: function() {
      var values;
      values = this.props.values;
      return {
        valueHasChanged: false,
        values: values.slice(0),
        focused: false
      };
    },
    componentDidMount: function() {
      var validation, values;
      validation = this.props.validation;
      values = this.state.values;
      this.inputId = makeGuid();
      this.validate(validation, values);
      this.focusBtn.addEventListener('focus', this.handleFocus);
      return this.focusBtn.addEventListener('blur', this.handleBlur);
    },
    componentWillReceiveProps: function(nextProps) {
      var i, j, len, ref2, ref3, ref4, returnFullObjects, validation, validationChanged, value, valueField, values;
      values = nextProps.values, validation = nextProps.validation, valueField = nextProps.valueField, returnFullObjects = nextProps.returnFullObjects;
      validationChanged = (typeof validation === 'function' ? false : !isEqual(validation != null ? validation.messages : void 0, (ref2 = this.props.validation) != null ? ref2.messages : void 0));
      if (validationChanged) {
        this.validate(validation, values);
      }
      if (this.state.values.length !== values.length) {
        this.setState({
          values: values
        });
        return this.validate(validation, values);
      } else {
        ref3 = this.state.values;
        for (i = j = 0, len = ref3.length; j < len; i = ++j) {
          value = ref3[i];
          if (returnFullObjects) {
            if (value[valueField] !== ((ref4 = values[i]) != null ? ref4[valueField] : void 0)) {
              break;
            }
          } else {
            if (value !== values[i]) {
              break;
            }
          }
        }
        if (i === this.state.values.length) {
          return;
        }
        this.setState({
          values: values
        });
        return this.validate(validation, values);
      }
    },
    componentWillUnmount: function() {
      this.context.clearValidationError({
        groupId: this.inputId,
        isInPopover: this.props.isInPopover
      });
      this.focusBtn.removeEventListener('focus', this.handleFocus);
      return this.focusBtn.removeEventListener('blur', this.handleBlur);
    },
    render: function() {
      var className, defaultPlaceholder, disabled, error, focused, forceShowAllErrors, getValidationStatus, isValid, labelField, options, placeholder, ref2, ref3, ref4, selected, tabIndex, value, valueField, valueHasChanged, values;
      ref2 = this.state, valueHasChanged = ref2.valueHasChanged, focused = ref2.focused, values = ref2.values;
      ref3 = this.props, options = ref3.options, tabIndex = ref3.tabIndex, disabled = ref3.disabled, valueField = ref3.valueField, labelField = ref3.labelField, tabIndex = ref3.tabIndex, placeholder = ref3.placeholder, defaultPlaceholder = ref3.defaultPlaceholder;
      getValidationStatus = this.context.getValidationStatus;
      ref4 = getValidationStatus(this.inputId), error = ref4.error, forceShowAllErrors = ref4.forceShowAllErrors;
      isValid = error == null;
      className = 'multiselect field-wrap';
      if (!isValid && (valueHasChanged || forceShowAllErrors)) {
        className += ' invalid';
      }
      if (disabled) {
        className += ' disabled';
      }
      if (focused) {
        className += ' focused';
      }
      selected = (function() {
        var j, len, results;
        results = [];
        for (j = 0, len = values.length; j < len; j++) {
          value = values[j];
          results.push(MultiSelectValue({
            key: value[valueField] || value,
            value: value,
            labelField: labelField,
            disabled: disabled,
            onRemove: this.onRemove
          }));
        }
        return results;
      }).call(this);
      if (selected.length === 0) {
        selected = div({
          key: 'placeholder',
          className: 'placeholder'
        }, placeholder != null ? placeholder : (typeof t !== "undefined" && t !== null ? t(defaultPlaceholder) : defaultPlaceholder));
      }
      return div({
        className: className,
        onClick: !disabled ? this.handleClick : null,
        ref: (function(_this) {
          return function(anchor) {
            _this.anchor = anchor;
          };
        })(this)
      }, [
        button({
          key: 'focusBtn',
          className: 'focus-btn',
          ref: (function(_this) {
            return function(focusBtn) {
              _this.focusBtn = focusBtn;
            };
          })(this),
          onKeyDown: this.handleKeydown,
          tabIndex: tabIndex,
          disabled: disabled
        }), selected, div({
          className: 'field-errors-show',
          key: 'textInputErrorsShow',
          ref: 'errorAnchor',
          onMouseOver: this.handleErrorMouseOver,
          onMouseOut: this.handleErrorMouseOut
        })
      ]);
    },
    getValue: function() {
      return this.state.values.slice(0);
    },
    handleClick: function() {
      var focused;
      if (this.pvrId != null) {
        return;
      }
      focused = this.state.focused;
      if (!focused) {
        this.focusBtn.focus();
      }
      return this.renderOptionList();
    },
    renderOptionList: function() {
      var closeOnSelect, labelField, openOverlay, options, ref2, returnFullObjects, valueField, values;
      openOverlay = this.context.openOverlay;
      ref2 = this.props, options = ref2.options, labelField = ref2.labelField, valueField = ref2.valueField, returnFullObjects = ref2.returnFullObjects, closeOnSelect = ref2.closeOnSelect;
      values = this.state.values;
      return this.pvrId = openOverlay({
        component: MultiSelectOptionList,
        id: 'popover',
        noBackdrop: true,
        disableBackgroundControls: false,
        overlayId: this.pvrId,
        props: {
          anchor: this.anchor,
          options: options,
          values: values,
          onRemove: this.onRemove,
          onAdd: this.onAdd,
          labelField: labelField,
          valueField: valueField,
          returnFullObjects: returnFullObjects,
          closeOnSelect: closeOnSelect,
          onClose: this.handlePvrClose
        }
      });
    },
    handleKeydown: function(arg) {
      var keyCode;
      keyCode = arg.keyCode;
      if ((keyCode !== SPACE && keyCode !== DOWN_ARROW) || (this.pvrId != null)) {
        return;
      }
      return this.handleClick();
    },
    handleChange: function() {
      var closeOnSelect, closeOverlay, jsonPath, onChange, ref2, validation, values;
      ref2 = this.props, jsonPath = ref2.jsonPath, onChange = ref2.onChange, validation = ref2.validation, closeOnSelect = ref2.closeOnSelect;
      closeOverlay = this.context.closeOverlay;
      values = this.getValue();
      if (typeof onChange === "function") {
        onChange(values, jsonPath);
      }
      this.validate(validation, values);
      if (closeOnSelect) {
        if (this.pvrId != null) {
          return closeOverlay(this.pvrId);
        }
      } else {
        if (this.pvrId != null) {
          return this.renderOptionList();
        }
      }
    },
    onAdd: function(newValue) {
      var ref2, returnFullObjects, valueField, valueHasChanged, values;
      values = this.state.values;
      ref2 = this.props, returnFullObjects = ref2.returnFullObjects, valueField = ref2.valueField;
      newValue = returnFullObjects ? newValue : newValue[valueField];
      values.push(newValue);
      valueHasChanged = true;
      return this.setState({
        values: values,
        valueHasChanged: valueHasChanged
      }, this.handleChange);
    },
    onRemove: function(removedValue) {
      var i, item, j, len, ref2, returnFullObjects, valueField, valueHasChanged, values;
      values = this.state.values;
      ref2 = this.props, valueField = ref2.valueField, returnFullObjects = ref2.returnFullObjects;
      for (i = j = 0, len = values.length; j < len; i = ++j) {
        item = values[i];
        if (!((returnFullObjects ? item[valueField] === removedValue[valueField] : item === removedValue[valueField]))) {
          continue;
        }
        values.splice(i, 1);
        break;
      }
      valueHasChanged = true;
      return this.setState({
        values: values,
        valueHasChanged: valueHasChanged
      }, this.handleChange);
    },
    handlePvrClose: function() {
      return setTimeout((function(_this) {
        return function() {
          return delete _this.pvrId;
        };
      })(this), 0);
    },
    handleFocus: function() {
      return this.setState({
        focused: true
      });
    },
    handleBlur: function() {
      return this.setState({
        focused: false
      });
    },
    validate: function(validation, value) {
      var addValidationError, clearValidationError, isInPopover, ref2, ref3, tabId, validationError;
      if (validation === false) {
        return;
      }
      if (typeof validation === 'function') {
        validationError = validation(value);
      } else {
        validationError = validation;
      }
      ref2 = this.props, isInPopover = ref2.isInPopover, tabId = ref2.tabId;
      ref3 = this.context, addValidationError = ref3.addValidationError, clearValidationError = ref3.clearValidationError;
      if (validationError != null) {
        return addValidationError({
          anchor: this.refs.errorAnchor,
          error: validationError,
          groupId: this.inputId,
          isInPopover: isInPopover,
          tabId: tabId
        });
      } else {
        return clearValidationError({
          groupId: this.inputId,
          isInPopover: isInPopover
        });
      }
    },
    handleErrorMouseOver: function() {
      var isInPopover;
      isInPopover = this.props.isInPopover;
      return this.context.toggleValidationError({
        groupId: this.inputId,
        status: true,
        isInPopover: isInPopover
      });
    },
    handleErrorMouseOut: function() {
      var isInPopover;
      isInPopover = this.props.isInPopover;
      return this.context.toggleValidationError({
        groupId: this.inputId,
        status: false,
        isInPopover: isInPopover
      });
    }
  });

  module.exports = MultiSelect;

}).call(this);
