React = require 'react'
createClass = require 'create-react-class'
PropTypes = require 'prop-types'
Pvr = React.createFactory(require('./pvr'))
map = require 'lodash/map'

MultiSelectOptionItem = require './multi_select_option_item'

{ul} = require 'react-dom-factories'

{ESCAPE, DOWN_ARROW, UP_ARROW, SPACE, TAB, ENTER} = require '../constants/keyboard'

MultiSelectOptionList = createClass

  displayName: 'MultiSelectOptionList'

  propTypes: 
    options: PropTypes.array.isRequired
    values: PropTypes.array.isRequired
    onRemove: PropTypes.func.isRequired
    onAdd: PropTypes.func.isRequired
    anchor: PropTypes.any
    close: PropTypes.func.isRequired
    onClose: PropTypes.func.isRequired
    valueField: PropTypes.string
    labelField: PropTypes.string
    returnFullObjects: PropTypes.bool
  
  getInitialState: ->
    focused: 0

  componentDidMount: ->
    @keyNav = off
    document.addEventListener 'keydown', @handleKeydown

  componentWillUnmount: ->
    {onClose} = @props

    document.removeEventListener 'keydown', @handleKeydown
    onClose()

  render: ->
    {focused} = @state
    {anchor, options, close, valueField, labelField, onAdd, onRemove} = @props
    {clientWidth} = anchor

    optionList = (MultiSelectOptionItem {
      key: optionItem[valueField] or optionItem
      option: optionItem
      isSelected: @optionIsSelected(optionItem)
      isFocused: i is focused
      onAdd
      onRemove
      labelField
      @focusOption
      index: i
    } for optionItem, i in options)
    
    width = clientWidth - (clientWidth * .1)

    Pvr {
      anchor: anchor
      width: if width < 470 then width else 470
      direction: 'below' 
      animateIn: no
      close: close
      height: 'auto'
      autoHeightClass: 'multiselect-option-list'
      nibColor: 'white'
      styleMixin:
        transition: 'top 300ms ease-out, left 300ms ease-out'
    }, ul {
        className: 'multiselect-option-list'
        onMouseMove: @handleMouseMove
        onKeyDown: @handleKeydown
        ref: (@listEl) =>
      }, optionList
  
  handleKeydown: (e) ->
    {close, options} = @props
    {keyCode} = e
    do e.preventDefault if keyCode in [TAB, SPACE]
    return do close if keyCode in [ESCAPE, SPACE]
    
    if keyCode in [UP_ARROW, DOWN_ARROW]
      do e.preventDefault
      {focused} = @state

      focused++ if keyCode is DOWN_ARROW
      focused-- if keyCode is UP_ARROW 

      lastIndex = options.length - 1

      focused = if focused < 0 then 0 else focused
      focused = if focused > lastIndex then lastIndex else focused

      @keyNav = on
      
      return @setState {focused}, @adjustScroll
    
    do @selectFocused if keyCode is ENTER

  selectFocused: ->
    {focused} = @state
    {options, onAdd, onRemove} = @props
    option = options[focused]

    if @optionIsSelected(option) then onRemove(option) else onAdd(option)
  
  focusOption: (focused) -> 
    # Only focus on hover when the user is moving the mouse
    @setState {focused} if @keyNav is off
  
  optionIsSelected: (option) ->
    {values, valueField, returnFullObjects} = @props
    
    value = option[valueField]

    if returnFullObjects then flatValues = map(values, valueField)
    else flatValues = values

    return yes for entry in flatValues when value is entry
    return no
  
  handleMouseMove: -> @keyNav = off

  adjustScroll: ->
    {scrollTop, clientHeight, children} = @listEl
    
    {focused} = @state
    focusedChild = children[focused]
    
    liHeight = focusedChild.clientHeight
    focusedTop = focusedChild.offsetTop
    focusedBottom = focusedTop + liHeight
    
    # At least a protion of the focused li is overflowing the bottom
    if focusedBottom > clientHeight + scrollTop
      @listEl.scrollTop = focusedBottom - clientHeight
    # At least a protion of the focused li is overflowing the top
    else if focusedTop < scrollTop
      @listEl.scrollTop = focusedTop

module.exports = React.createFactory(MultiSelectOptionList)